/*
 * (c) 2017, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include "proto.h"

extern int      verbose;
int             pulse_coid = -1;    // who to deliver the pulse to
int             rcvid      = -1;    // who to deliver the pulse to
struct sigevent pulse_event;        // will contain the pulse to be delivered


int gpio_devctl( resmgr_context_t *ctp, io_devctl_t *msg, gpio_ocb_t *ocb )
{
    int             status;
    int             nbytes      = 0;
    gpio_devctl_t   *devmsg_ptr = NULL;
    uint8_t         pin;
    gpio_dev_t      *dev        = ocb->hdr.attr;

    if ( (status = iofunc_devctl_default( ctp, msg, &ocb->hdr )) != _RESMGR_DEFAULT )
        return (_RESMGR_ERRNO( status ));
    status=0;

    switch ( msg->i.dcmd )
    {
        case DCMD_SET_INPUT:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );
            pin        = devmsg_ptr->set_input.pin_num;

            status = hw_set_input( dev->hdl, pin );
            if ( status < 0 )
                return (EINVAL);
            break;

        case DCMD_SET_OUTPUT:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );
            pin        = devmsg_ptr->set_output.pin_num;

            status = hw_set_output( dev->hdl, pin );
            if ( status < 0 )
                return (EINVAL);
            break;

        case DCMD_READ:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );

            status = hw_cmd_read( dev->hdl, (void *)devmsg_ptr );
            if ( status != 1 )
                return (EINVAL);

            msg->o.ret_val = 0;
            return (_RESMGR_PTR( ctp,  &msg->o, sizeof( msg->o ) + sizeof( gpio_devctl_read_t ) ));
            break;

        case DCMD_WRITE:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );

            status = hw_cmd_write( dev->hdl, (void*)devmsg_ptr );
            if ( status != 1 )
                return (EINVAL);

            msg->o.ret_val = 0;
            return (_RESMGR_PTR( ctp,  &msg->o, sizeof( msg->o ) + sizeof( gpio_devctl_write_t ) ));
            break;

        case DCMD_INT_ENABLE:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );
            pin        = devmsg_ptr->cmd_intterupt.pin_num;

//            status = hw_int_enable_wlevel( dev->hdl, pin, devmsg_ptr->cmd_intterupt.level );

            if ( hw_int_enable_wlevel( dev->hdl, pin, devmsg_ptr->cmd_intterupt.level ) != 0 )
                return (EINVAL);
             break;

        case DCMD_INT_DISABLE:
            // get a pointer to the structure
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );
            pin        = devmsg_ptr->cmd_intterupt.pin_num;

            if ( hw_set_output( dev->hdl, pin ) < 0 )
            {
                gpio_slogf( "GPIO RM: GPIO pin %d interrupt disable failed ", pin );
                return (EINVAL);
            }

            if ( hw_int_disable( dev->hdl, pin ) < 0 )
            {
                gpio_slogf( "GPIO RM: GPIO pin %d interrupt disable failed ", pin );
                return (EINVAL);
            }
            break;

        case DCMD_INT_ENABLE_WAIT_PULSE:
            devmsg_ptr = (gpio_devctl_t*)_DEVCTL_DATA( msg->i );
            rcvid      = ctp->rcvid;
            pulse_coid = devmsg_ptr->cmd_intterupt.coid;
            pin        = devmsg_ptr->cmd_intterupt.pin_num;

            if ( hw_int_enable_wlevel( dev->hdl, pin, devmsg_ptr->cmd_intterupt.level ) != 0 )
            {
                gpio_slogf( "GPIO RM: GPIO pin %d interrupt enable failed ", pin );
                return (EINVAL);
            }

            if ( verbose )
                gpio_slogf( "GPIO RM: Interrupt PIN %d recived pulse_coid %d rcvid %d ",
                            pin, pulse_coid, rcvid );

            return (_RESMGR_NPARTS( 0 ));
            break;

        default:
            return (ENOSYS);
    }

    _IO_SET_WRITE_NBYTES( ctp, nbytes );
    return (_RESMGR_NPARTS( 0 ));
}
