/*
 * (c) 2017, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#include "proto.h"


int                         verbose = 0;
static volatile unsigned    done    = 0;
static gpio_dev_t           *Dev;
extern gpio_dev_t *am35_hw_init( gpio_dev_t *dev );
void am35_hw_fini( gpio_dev_t *dev );

void *interrupt_thread( void *data );
struct sigevent int_event;  // the SIGEV_INTR event the ISR will return

extern int                  pulse_coid;     // who to deliver the pulse to
extern int                  rcvid;          // who to deliver the pulse to
extern struct sigevent      pulse_event;    // will contain the pulse to be delivered


int gpio_slogf( const char *fmt, ... )
{
    int         status;
    va_list     arglist;

    if ( verbose > 2 )
    {
        va_start( arglist, fmt );
        vfprintf( stderr, fmt, arglist );
        va_end( arglist );
        fprintf( stderr, "\n" );
    }

    va_start( arglist, fmt );
    status = vslogf( _SLOGC_GPIO, _SLOG_INFO, fmt, arglist );
    va_end( arglist );

    return status;
}


static void gpio_exit( int signo )
{
    atomic_set( &done, 1 );
    dispatch_unblock( Dev->ctp );
}


static int gpio_main( gpio_dev_t *dev )
{
    resmgr_connect_funcs_t  connect_funcs;
    resmgr_io_funcs_t       io_funcs;
    resmgr_attr_t           rattr;
    iofunc_funcs_t          ocb_funcs = { _IOFUNC_NFUNCS, gpio_ocb_calloc, gpio_ocb_free };
    iofunc_mount_t          mount     = { 0, 0, 0, 0, &ocb_funcs };
    dispatch_context_t      *ctp;
    char                    devname[PATH_MAX + 1];
    struct sigaction        sa;

    if ( !dev )
        return (-1);

    Dev = dev;

    if ( NULL == (dev->dpp = dispatch_create()) )
    {
        gpio_slogf( "GPIO RM: dispatch_create failed: %s", strerror( errno ) );
        perror( "GPIO RM: dispatch_create() failed" );
        return (-1);
    }

    /* set up i/o handler functions */
    memset( &rattr, 0, sizeof( rattr ) );
    rattr.nparts_max   = GPIO_RESMGR_NPARTS_MIN;
    rattr.msg_max_size = GPIO_RESMGR_MSGSIZE_MIN;

    iofunc_func_init( _RESMGR_CONNECT_NFUNCS, &connect_funcs, _RESMGR_IO_NFUNCS, &io_funcs );
    io_funcs.devctl = gpio_devctl;

    iofunc_attr_init( &dev->hdr, S_IFCHR | 0666, NULL, NULL );
    dev->hdr.mount = &mount;

    /* register device name */
    snprintf( devname, PATH_MAX, GPIO_DEVICE_NAME );

    if ( -1 == (dev->id = resmgr_attach( dev->dpp, &rattr, devname, _FTYPE_ANY, 0,
                                         &connect_funcs, &io_funcs, dev )) )
    {
        gpio_slogf( "GPIO RM: resmgr_attach failed: %s", strerror( errno ) );
        perror( "GPIO RM: resmgr_attach() failed" );
        return (-1);
    }

    if ( NULL == (dev->ctp = dispatch_context_alloc( dev->dpp )) )
    {
        gpio_slogf( "GPIO RM: dispatch_context_alloc failed: %s", strerror( errno ) );
        perror( "GPIO RM: dispatch_context_alloc() failed" );
        return (-1);
    }

    /* register exit handler */
    sigemptyset( &sa.sa_mask );
    sigaddset( &sa.sa_mask, SIGTERM );
    sa.sa_handler = gpio_exit;
    sa.sa_flags   = 0;
    sigaction( SIGTERM, &sa, NULL );

    if ( -1 == pthread_create( NULL, NULL, interrupt_thread, NULL ) )
    {
        gpio_slogf( "GPIO RM: pthread_create failed: %s", strerror( errno ) );
        perror( "GPIO RM: pthread_create() failed" );
        return (-1);
    }

    /* background the process */
    procmgr_daemon( 0, PROCMGR_DAEMON_NOCLOSE | PROCMGR_DAEMON_NODEVNULL );

    /* wait for messages */
    while ( !done )
    {
        if ( ctp = dispatch_block( dev->ctp ) )
        {
            dev->ctp = ctp;
            dispatch_handler( dev->ctp );
        } else
            if ( errno != EFAULT )
                atomic_set( &done, 1 );
    }

    /* Received SIGTERM: clean up */
    resmgr_detach( dev->dpp, dev->id, _RESMGR_DETACH_ALL );

    return (0);
}


void * interrupt_thread( void *data )
{
    struct sigevent     event;      // pulse
    gpio_dev_t          *dev = Dev;
    gpio_interrupt_t    gpio_int;
    int                 pin  = -1;
    struct sigevent     gpio1_int_event;
    struct sigevent     gpio2_int_event;
    struct sigevent     gpio3_int_event;
    struct sigevent     gpio4_int_event;
    struct sigevent     gpio5_int_event;
    struct sigevent     gpio6_int_event;

    if ( ThreadCtl( _NTO_TCTL_IO, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: ThreadCtl failed: %s", strerror( errno ) );
        perror( "GPIO RM: ThreadCtl" );
        exit( EXIT_FAILURE );
    }

    // set up a SIGEV_INTR event structure to for the ISR to return, the ISR
    // will deliver this event after every 1000th interrupt.  When
    // it does, InterruptWait() will unblock.

    // this macro fills in the event structure with SIGEV_INTR
    SIGEV_INTR_INIT( &gpio1_int_event );
    SIGEV_INTR_INIT( &gpio2_int_event );
    SIGEV_INTR_INIT( &gpio3_int_event );
    SIGEV_INTR_INIT( &gpio4_int_event );
    SIGEV_INTR_INIT( &gpio5_int_event );
    SIGEV_INTR_INIT( &gpio6_int_event );

    if ( InterruptAttachEvent( OMAP3530_GPIO_6_IRQ, &gpio6_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }
    if ( InterruptAttachEvent( OMAP3530_GPIO_1_IRQ, &gpio1_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }
    if ( InterruptAttachEvent( OMAP3530_GPIO_2_IRQ, &gpio2_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }
    if ( InterruptAttachEvent( OMAP3530_GPIO_3_IRQ, &gpio3_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }
    if ( InterruptAttachEvent( OMAP3530_GPIO_4_IRQ, &gpio4_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }
    if ( InterruptAttachEvent( OMAP3530_GPIO_5_IRQ, &gpio5_int_event, 0 ) == -1 )
    {
        gpio_slogf( "GPIO RM: InterruptAttachEvent failed: %s", strerror( errno ) );
        exit( EXIT_FAILURE );
    }

    while ( 1 )
    {
        InterruptWait( 0, NULL );

        gpio_slogf( "GPIO RM: unblocked from InterruptWait()" );

        pin = hw_irq_status( dev->hdl, &gpio_int );
        if ( pin != -1 )
            gpio_slogf( "GPIO RM: Interrupt from PIN %d (level = %d)", gpio_int.pin, gpio_int.level );
        else
            continue;

        if ( rcvid != -1 )
        {
            if ( verbose > 2 )
            {
                gpio_slogf( "GPIO RM delivering pulse" );
                gpio_slogf( "GPIO RM: pulse_coid %d rcvid %d", pulse_coid, rcvid );
            }

            // fill in the event structure for a pulse 
            SIGEV_PULSE_INIT( &event, pulse_coid, SIGEV_PULSE_PRIO_INHERIT, _PULSE_CODE_MINAVAIL, pin );

            if ( MsgDeliverEvent( rcvid, &event ) == -1 )
            {
                gpio_slogf( "GPIO RM: MsgDeliverEvent failed: %s", strerror( errno ) );
                perror( "GPIO RM: MsgDeliverEvent" );
                rcvid = -1;
            }

//            if ( MsgSendPulse( pulse_coid, -1, _PULSE_CODE_MINAVAIL, 0 ) == -1 )
//            {
//                gpio_slogf( "GPIO RM: MsgSendPulse failed: %s", strerror( errno ) );
//                perror( "GPIO RM: MsgSendPulse" );
//            }

            if ( verbose > 2 )
                gpio_slogf( "GPIO RM: pulse sent" );

            if ( hw_int_enable_wlevel( dev->hdl, gpio_int.pin, gpio_int.level ) != 0 )
                gpio_slogf( "GPIO RM GPIO pin %d interrupt enable failed", gpio_int.pin );

            if ( InterruptUnmask( OMAP3530_GPIO_6_IRQ, 0 ) == -1 )
            {
                gpio_slogf( "GPIO RM: InterruptUnmask failed: %s", strerror( errno ) );
                perror( "GPIO RM: InterruptUnmask" );
            }
        }
    }
}


int main( int argc, char *argv[] )
{
    int             status;
    int             opt;
    gpio_dev_t      dev;

    memset( &dev, 0, sizeof( dev ) );

    while ( (opt = getopt( argc, argv, "v" )) != -1 )
    {
        switch ( opt )
        {
            case 'v':
                verbose++;
                break;
        }
    }

    gpio_slogf( "GPIO RM: starting GPIO resource manager" );

    am35_hw_init( &dev );
    status = gpio_main( &dev );
    am35_hw_fini( &dev );

    return status;
}
